import os
import unittest
import copy
from collections import namedtuple

import mock
from fedora_messaging.api import Message
from fedora_messaging.testing import mock_sends

from robosignatory.coreos import CoreOSSigner


TEST_CONFIG = {
    "signing": {
        "backend": "echo",
    },
    "koji_instances": {},
    "ostree_refs": {},
    "coreos": {
        "bucket": "testing",
        "key": "testing",
        "aws": {
            "access_key": "testing",
            "access_secret": "testing",
            "region": "us-east-1",
        }
    },
}

ARTIFACTS_MESSAGE = Message(
    topic="org.fedoraproject.prod.coreos.build.request.artifacts-sign",
    body={
        "build_id": "buildid",
        "stream": "stream",
        "basearch": "basearch",
        "artifacts": [{
            "file": "s3://host/some/path/test1",
            "checksum": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
        }]
    }
)

OSTREE_MESSAGE = Message(
    topic="org.fedoraproject.prod.coreos.build.request.ostree-sign",
    body={
        "build_id": "buildid",
        "stream": "stream",
        "basearch": "basearch",
        "commit_object": "s3://host/some/path/test1",
        "checksum": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
    }
)

S3Object = namedtuple("S3Object", "size")

def fake_download(source, local):
    open(local, "w").close()

def fake_download_and_artifact_sign(source, local):
    fake_download(source, local)
    # Also create the sig file for testing
    open(local + ".sig", "w").close()

def fake_download_and_ostree_sign(source, local):
    fake_download(source, local)
    # Also create the sig file for testing
    open(os.path.dirname(local) + "/ostree-commitmeta-object", "w").close()


class TestCoreOS(unittest.TestCase):

    def setUp(self):
        self.consumer = CoreOSSigner(TEST_CONFIG)
        self.consumer.bucket = mock.Mock()

    def _get_response_message(self, source_msg, failed=False):
        return Message(
            topic=source_msg.topic + ".finished",
            body={
                "build_id": source_msg.body["build_id"],
                "stream": source_msg.body["stream"],
                "basearch": source_msg.body["basearch"],
                "status": "FAILURE" if failed else "SUCCESS",
            }
        )

    @mock.patch('robosignatory.coreos.utils.run_command')
    def test_artifacts_sign(self, run_command):
        self.consumer.bucket.download_file.side_effect = fake_download_and_artifact_sign
        run_command.return_value = 0, "", ""
        self.consumer.bucket.objects.filter.return_value = [S3Object(size=0)]
        expected_response = self._get_response_message(ARTIFACTS_MESSAGE)

        with mock_sends(expected_response):
            self.consumer.consume(ARTIFACTS_MESSAGE)

        self.consumer.bucket.download_file.assert_called()
        assert self.consumer.bucket.download_file.call_args_list[0][0][0] == "some/path/test1"
        run_command.assert_called()
        self.consumer.bucket.upload_file.assert_called()
        assert self.consumer.bucket.upload_file.call_args_list[0][0][1] == "some/path/test1.sig"

    @mock.patch('robosignatory.coreos.utils.run_command')
    def test_ostree_sign(self, run_command):
        self.consumer.bucket.download_file.side_effect = fake_download_and_ostree_sign
        run_command.return_value = 0, "", ""
        self.consumer.bucket.objects.filter.return_value = [S3Object(size=0)]
        expected_response = self._get_response_message(OSTREE_MESSAGE)

        with mock_sends(expected_response):
            self.consumer.consume(OSTREE_MESSAGE)

        self.consumer.bucket.download_file.assert_called()
        assert self.consumer.bucket.download_file.call_args_list[0][0][0] == "some/path/test1"
        run_command.assert_called()
        self.consumer.bucket.upload_file.assert_called()
        assert self.consumer.bucket.upload_file.call_args_list[0][0][1] == "some/path/ostree-commitmeta-object"

    @mock.patch('robosignatory.coreos.utils.run_command')
    def test_wrong_checksum(self, run_command):
        new_body = copy.deepcopy(ARTIFACTS_MESSAGE.body)
        new_body["artifacts"][0]["checksum"] = "wrong-checksum"
        msg = Message(topic=ARTIFACTS_MESSAGE.topic, body=new_body)
        self.consumer.bucket.download_file.side_effect = fake_download
        expected_response = self._get_response_message(ARTIFACTS_MESSAGE, failed=True)

        with mock_sends(expected_response):
            self.consumer.consume(msg)

        self.consumer.bucket.download_file.assert_called()
        run_command.assert_not_called()
        self.consumer.bucket.upload_file.assert_not_called()

    @mock.patch('robosignatory.coreos.utils.run_command')
    def test_signing_failed(self, run_command):
        self.consumer.bucket.download_file.side_effect = fake_download
        run_command.return_value = 1, "stdout", "stderr"
        expected_response = self._get_response_message(ARTIFACTS_MESSAGE, failed=True)

        with mock_sends(expected_response):
            self.consumer.consume(ARTIFACTS_MESSAGE)

        self.consumer.bucket.download_file.assert_called()
        run_command.assert_called()
        self.consumer.bucket.upload_file.assert_not_called()

    @mock.patch('robosignatory.coreos.utils.run_command')
    def test_no_signature(self, run_command):
        self.consumer.bucket.download_file.side_effect = fake_download
        run_command.return_value = 0, "stdout", "stderr"
        expected_response = self._get_response_message(ARTIFACTS_MESSAGE, failed=True)

        with mock_sends(expected_response):
            self.consumer.consume(ARTIFACTS_MESSAGE)

        self.consumer.bucket.download_file.assert_called()
        run_command.assert_called()
        self.consumer.bucket.upload_file.assert_not_called()
